@@ -3,10 +3,10 @@
 ##
 ##  Anderson-Darling test and null distribution
 ##
-## $Revision: 1.6 $ $Date: 2014/06/24 02:12:20 $
+## $Revision: 1.10 $ $Date: 2018/06/06 08:25:51 $
 ##
 
-ad.test <- function(x, null="punif", ..., nullname) {
+ad.test <- function(x, null="punif", ..., estimated=FALSE, nullname) {
   xname <- deparse(substitute(x))
   nulltext <- deparse(substitute(null))
   if(is.character(null)) nulltext <- null
@@ -18,32 +18,35 @@ ad.test <- function(x, null="punif", ..., nullname) {
   stopifnot(is.numeric(x))
   x <- as.vector(x)
   n <- length(x)
-  F0 <- if(is.function(null)) null else
-        if(is.character(null)) get(null, mode="function") else
-        stop("Argument 'null' should be a function, or the name of a function")
+  F0 <- getCdf(null)
   U <- F0(x, ...)
   if(any(U < 0 | U > 1))
     stop("null distribution function returned values outside [0,1]")
-  U <- sort(U)
-  k <- seq_len(n)
-  ## call Marsaglia C code
-  z <- .C(CgofADtestR,
-          x = as.double(U),
-          n = as.integer(n),
-          adstat = as.double(numeric(1)),
-          pvalue = as.double(numeric(1)),
-	  PACKAGE="goftest"
-          )
-  STATISTIC <- z$adstat
-  names(STATISTIC) <- "An"
-  PVAL <- z$pvalue
+  
+  #' perform test
+  if(!estimated || n <= 4) {
+    #' simple null hypothesis
+    z <- simpleADtest(U)
+    ADJUST <- NULL
+  } else {  
+    #' composite - use Braun (1980)
+    m <- round(sqrt(n))
+    z <- braun(U, simpleADtest, m=m)
+    ADJUST <- paste("Braun's adjustment using", m, "groups")
+  }
+  PVAL             <- z$pvalue
+  STATISTIC        <- z$statistic
+  names(STATISTIC) <- z$statname
+  
+  #' dress up
   METHOD <- c("Anderson-Darling test of goodness-of-fit",
+              ADJUST,
               paste("Null hypothesis:", nullname))
   extras <- list(...)
   parnames <- intersect(names(extras), names(formals(F0)))
   if(length(parnames) > 0) {
     pars <- extras[parnames]
-    pard <- character(0)
+    pard <- character(length(parnames))
     for(i in seq_along(parnames))
       pard[i] <- paste(parnames[i], "=", paste(pars[[i]], collapse=" "))
     pard <- paste("with",
@@ -52,6 +55,11 @@ ad.test <- function(x, null="punif", ..., nullname) {
                   paste(pard, collapse=", "))
     METHOD <- c(METHOD, pard)
   }
+  
+  coda <- paste("Parameters assumed to",
+                if(estimated) "have been estimated from data" else "be fixed")
+  METHOD <- c(METHOD, coda)
+  
   out <- list(statistic = STATISTIC,
                p.value = PVAL,
                method = METHOD,
@@ -60,6 +68,20 @@ ad.test <- function(x, null="punif", ..., nullname) {
   return(out)
 }
 
+simpleADtest <- function(U) {
+  ## Internal: call Marsaglia C code
+  U <- sort(U)
+  n <- length(U)
+  z <- .C(CgofADtestR,
+          x = as.double(U),
+          n = as.integer(n),
+          adstat = as.double(numeric(1)),
+          pvalue = as.double(numeric(1)),
+	  PACKAGE="goftest"
+          )
+  return(list(statistic=z$adstat, pvalue=z$pvalue, statname="An"))
+}
+
 pAD <- function(q, n=Inf, lower.tail=TRUE, fast=TRUE) {
   q <- as.numeric(q)
   p <- rep(NA_real_, length(q))
